
import 'package:collection/collection.dart';
import 'package:nextcloud_reminder/types/repeat.dart';

class TaskExtra extends Task {
  TaskExtra({required super.title,
    super.done = false,
    super.contexts = const [],
    super.projects = const [],
    super.meta = const {},
    super.priority,
    super.begin,
    super.end,
    this.lineNumber,
    this.repeat = const [],
  });

  TaskExtra.fromTask(Task t, {this.lineNumber})
      : repeat = t.meta["repeat"]?.split("/").map(RepeatInterval.fromString).whereNotNull().toList() ?? []
      , super(title: t.title,
              done: t.done,
              contexts: t.contexts,
              projects: t.projects,
              meta: t.meta,
              priority: t.priority,
              begin: t.begin,
              end: t.end,
            );

  final int? lineNumber;
  final List<RepeatInterval> repeat;

  formatAsTask() {
    return super.toString();
  }

  @override
  String toString() {
    return "$lineNumber: ${super.toString()}";
  }
}

class Task {
  final bool done;
  final DateTime? begin;
  final DateTime? end;
  final String title;
  final List<String> contexts;
  final List<String> projects;
  final Map<String,String> meta;
  final String? priority;

  Task({
    required this.title,
    this.done = false,
    this.contexts = const [],
    this.projects = const [],
    this.meta = const {},
    this.priority,
    this.begin,
    this.end,
  });

  static String formatDate(DateTime dt) {
    return "${dt!.year}-${dt!.month.toString().padLeft(2,'0')}-${dt!.day.toString().padLeft(2,'0')}";
  }

  @override
  String toString() {
    return (done ? "x " : "")
         + (priority == null ? "" : "(${priority!}) ")
         + (begin == null ? "" : "${Task.formatDate(begin!)} ")
         + (end == null ? "" : "${Task.formatDate(end!)} ")
         + ("$title ")
         + meta.entries.map((entry) => "${entry.key}:${entry.value}").join(" ")
    ;
  }
}